//
//  AppDelegate.m
//  SMB MenuExtra
//
//  Created by John Slaughter on 3/1/13.
//  Copyright (c) 2013 Engineering & Science Computing - University of Notre Dame.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "AppDelegate.h"
#import "ConnectWindowController.h"
#import "PrefsWindowController.h"
#import "AutoMountController.h"
#import "SharedFunctions.h"
#import "ClickHandlerWindowController.h"

@implementation AppDelegate

@synthesize myConnectWindowController = _myConnectWindowController;
@synthesize myPrefsWindowController = _myPrefsWindowController;
@synthesize myClickHandlerWindowController = _myClickHandlerWindowController;
@synthesize statusMenu = _statusMenu;
@synthesize statusObject = _statusObject;
@synthesize fore_image = _fore_image;
@synthesize back_image = _back_image;


#pragma mark - Standard Methods

- (id)init
{
    self = [super init];
    if (self) {
        NSBundle *bundle = [NSBundle mainBundle];
        
        // Test for Retina.
        CGFloat scale = 1.0f;
        if ([[NSScreen mainScreen] respondsToSelector:@selector(backingScaleFactor)])
        {
            scale = [[NSScreen mainScreen] backingScaleFactor];
            NSDebug(@"Background Scale is: %.0f", scale);
        } else {
            // If we don't know what "backingScaleFactor" is, then we're too old for Retina anyway.
            scale = 1;
        }
        
        if (scale == 2.000000) {
            _fore_image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"drive@2x" ofType:@"png"]];
            _back_image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"drive@2x" ofType:@"png"]];
        } else {
            _fore_image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"drive" ofType:@"png"]];
            _back_image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"drive" ofType:@"png"]];
        }
       
        // Set up event handler for URL types declared in project info plist.
        [[NSAppleEventManager sharedAppleEventManager] setEventHandler:self andSelector:@selector(handleURLEvent:withReplyEvent:) forEventClass:kInternetEventClass andEventID:kAEGetURL];

    }
    return self;
}


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{

}


- (BOOL)application:(NSApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    return YES;
}


- (void) awakeFromNib
{
    // Application starts, build a MenuExtra item.
    
    self.statusObject = [[NSStatusBar systemStatusBar] statusItemWithLength:NSVariableStatusItemLength];
    [self.statusObject setImage:self.fore_image];
    [self.statusObject setAlternateImage:self.back_image];
	[self.statusObject setMenu:self.statusMenu];
	[self.statusObject setToolTip:@"SMB Server MenuExtra"];
	[self.statusObject setHighlightMode:YES];
    
    
    // Then see if there are any saved servers we're supposed to auto-mount at startup.
    
    AutoMountController *theAutoMount = [[AutoMountController alloc] init];
    [theAutoMount connectAllAutoLoginServers];
    [theAutoMount release];

}


-(void)dealloc
{
    [_fore_image release];
    [_back_image release];
    [_statusObject release];
    [_myConnectWindowController release];
    [super dealloc];
}


#pragma mark - Event Handler Methods

- (void)handleURLEvent:(NSAppleEventDescriptor*)event withReplyEvent:(NSAppleEventDescriptor*)replyEvent
{
    // Received a URL handler request.  Do something with it.
    
    NSString* url = [[event paramDescriptorForKeyword:keyDirectObject] stringValue];
    NSDebug(@"Received handler request for: '%@'", url);
    
    _myClickHandlerWindowController = [[ClickHandlerWindowController alloc] initWithServer: url];
    
	[_myClickHandlerWindowController showWindow:self];
	ProcessSerialNumber psn = { 0, kCurrentProcess };
	SetFrontProcess(&psn);
}


#pragma mark - Menu Item Methods

- (IBAction)connectToSMBServer:(id)sender
{    
    if (! self.myConnectWindowController ) {
		_myConnectWindowController	= [[ConnectWindowController alloc] init];
	}
	[_myConnectWindowController showWindow:self];
	ProcessSerialNumber psn = { 0, kCurrentProcess };
	SetFrontProcess(&psn);
}


- (IBAction)openPrefs:(id)sender
{    
    if (! self.myPrefsWindowController ) {
		_myPrefsWindowController	= [[PrefsWindowController alloc] init];
	}
	[_myPrefsWindowController showWindow:self];
	ProcessSerialNumber psn = { 0, kCurrentProcess };
	SetFrontProcess(&psn);
}


- (IBAction)mountAllSavedVolumes:(id)sender
{    
    AutoMountController *theAutoMount = [[AutoMountController alloc] init];
    [theAutoMount connectAllSavedServers];
    [theAutoMount release];
}


- (IBAction)mountAllCheckedVolumes:(id)sender
{
    AutoMountController *theAutoMount = [[AutoMountController alloc] init];
    [theAutoMount connectAllAutoLoginServers];
    [theAutoMount release];
}


@end
