//
//  AssociateWithSMBURLs.m
//  SMB MenuExtra
//
//  Created by John Slaughter on 3/17/13.
//  Copyright (c) 2013 Engineering & Science Computing - University of Notre Dame.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "AssociateWithSMBURLs.h"

@implementation AssociateWithSMBURLs


- (void) setAssociateWithSMBURLs: (BOOL) enabled
{
    if (enabled == 0) {

        if ([self handlerExistsWithID:@"edu.nd.esc.smb-menuextra"]) {
            [self deleteHandlerWithID:@"edu.nd.esc.smb-menuextra"];
        }
    
    } else if (enabled == 1) {
        
        //  This setting is saved in the following file:
        //  ~/Library/Preferences/com.apple.LaunchServices.plist
        
        CFStringRef bundleID = (CFStringRef)[[NSBundle mainBundle] bundleIdentifier];
        OSStatus smbResult = LSSetDefaultHandlerForURLScheme(CFSTR("smb"), bundleID);
        if (smbResult) {
            // Error.
            NSDebug(@"Set SMB Handler error: %d", (int)smbResult);
        } else {
            // Success.
            NSDebug(@"Set SMB Handler success No: %d", (int)smbResult);
        }
    }
}


- (BOOL) associateWithSMBURLs
{    
    return [self handlerExistsWithID:@"edu.nd.esc.smb-menuextra"];
}

- (void) deleteHandlerWithID:(NSString*) theID
{
    NSDictionary *theHandlersPlist = [NSDictionary dictionaryWithContentsOfFile:[@"~/Library/Preferences/com.apple.LaunchServices.plist" stringByExpandingTildeInPath]];
    NSArray *theHandlerArray = [theHandlersPlist objectForKey:@"LSHandlers"];
    
    // I know...  There are better and faster ways to do this, but I needed the array index value while looping.
    int i;
    for (i = 0; i < [theHandlerArray count]; i++) {
        
        if ([[[theHandlerArray objectAtIndex:i] objectForKey:@"LSHandlerRoleAll"] isEqualToString:theID]) {
            
            NSTask *writeServerSetting=[[NSTask alloc] init];
            [writeServerSetting setLaunchPath: @"/usr/libexec/PlistBuddy"];
            [writeServerSetting setArguments:[NSArray arrayWithObjects:@"-c", [NSString stringWithFormat:@"Delete :LSHandlers:%d dict", i], [@"~/Library/Preferences/com.apple.LaunchServices.plist" stringByExpandingTildeInPath], nil]];
            
            NSPipe *pipeSetting = [NSPipe pipe];
            [writeServerSetting setStandardOutput: pipeSetting];
            [writeServerSetting setStandardError: pipeSetting];
            [writeServerSetting launch];
            
            [writeServerSetting waitUntilExit];
            
            NSData *outputData = [[pipeSetting fileHandleForReading] readDataToEndOfFile];
            NSString *outputString = [[NSString alloc] initWithData:outputData encoding:NSUTF8StringEncoding];
            NSDebug(@"Task Output: %@", outputString);
            
            [outputString release];
            [writeServerSetting release];
        } else {
            NSDebug(@"Handler was not found at index:%d.", i);
        }
    }
}


- (BOOL) handlerExistsWithID:(NSString*) theID
{
    // Iterate through the array, looking for the value 'edu.nd.esc.smb-menuextra' in the key 'LSHandlerRoleAll'
    
    BOOL theBool = FALSE;
    
    NSDictionary *theHandlersPlist = [NSDictionary dictionaryWithContentsOfFile:[@"~/Library/Preferences/com.apple.LaunchServices.plist" stringByExpandingTildeInPath]];
    NSArray *theHandlerArray = [theHandlersPlist objectForKey:@"LSHandlers"];
    
    for (id object in theHandlerArray) {
        if ([[object objectForKey:@"LSHandlerRoleAll"] isEqualToString:theID]) {
            theBool = TRUE;
        }
    }
    return theBool;
}


@end
