//
//  AutoMountController.m
//  SMB MenuExtra
//
//  Created by John Slaughter on 3/1/13.
//  Copyright (c) 2013 Engineering & Science Computing - University of Notre Dame.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "AutoMountController.h"
#import "RHKeychain.h"
#import "SharedFunctions.h"

@implementation AutoMountController

@synthesize defaults = _defaults;
@synthesize arrayServers = _arrayServers;


#pragma mark - Standard Methods

- (id) init
{
	self = [super init];
    if (self) {
        
        // Init the User Defaults.
        _defaults = [NSUserDefaults standardUserDefaults];
        
        // Init the array of servers from the preferences.
        if ([self.defaults objectForKey:@"savedServers"] != nil) {
            _arrayServers  = [[NSMutableArray alloc] initWithArray:[_defaults arrayForKey:@"savedServers"]];
        }
    }
	return self;
}


#pragma mark - Connection Methods

- (void)ConnectToServer:(NSString *)theServer asUser:(NSString *)theUser withPassword:(NSString *)thePassword
{
    if ([thePassword isNotEqualTo:@""]) {
                
        int RandomIndex = arc4random() % 100000;
        NSString * tempPath = [NSTemporaryDirectory() stringByAppendingPathComponent: [NSString stringWithFormat:@"Share_%d", RandomIndex]];
        NSDebug(@"tempPath: %@", tempPath);
        
        NSTask *makeTmp=[[[NSTask alloc] init] autorelease];
        [makeTmp setLaunchPath: @"/bin/mkdir"];
        [makeTmp setArguments:[NSArray arrayWithObjects:tempPath, nil]];
        [makeTmp launch];
        
        NSTask *mountFS=[[NSTask alloc] init];
        [mountFS setLaunchPath: @"/sbin/mount"];
        [mountFS setArguments:[NSArray arrayWithObjects:@"-t", @"smbfs", [NSString stringWithFormat:@"cifs://%@:%@@%@", sanitizeString(theUser), sanitizeString(thePassword), sanitizeString(removeSMBPrefix(theServer))], tempPath, nil]];
        
        NSPipe *pipeEthernet = [NSPipe pipe];
        [mountFS setStandardOutput: pipeEthernet];
        [mountFS setStandardError: pipeEthernet];
        [mountFS launch];
        [mountFS waitUntilExit];
        
        NSData *outputData = [[pipeEthernet fileHandleForReading] readDataToEndOfFile];
        NSString *outputString = [[NSString alloc] initWithData:outputData encoding:NSUTF8StringEncoding];
        NSDebug(@"NSTask Output: %@", outputString);
        
        [outputString release];
        [mountFS release];
    }
}

- (void)connectAllSavedServers
{
    for (id object in _arrayServers) {
        NSDebug(@"Autologin for server: %@ (%@)", [object objectForKey:@"servername"], [object objectForKey:@"serviceName"]);
        BOOL theBool = [[object objectForKey:@"savePass"] boolValue];
        NSDebug(@"Save Pass status: %@", [object objectForKey:@"savePass"]);
        if (theBool == YES) {
            
            NSString *serverVar;
            if ([[object objectForKey:@"servername"] hasPrefix:[@"smb://" lowercaseString]]) {
                serverVar = [[object objectForKey:@"servername"] substringFromIndex: 6];
            } else {
                serverVar = [object objectForKey:@"servername"];
            }
            
            if ([[object objectForKey:@"serviceName"] hasPrefix: serverVar]) {
            
                [self ConnectToServer:[object objectForKey:@"servername"] asUser:[object objectForKey:@"userID"] withPassword: RHKeychainGetGenericPassword(NULL, [object objectForKey:@"serviceName"])];
            }
        }
    }
}

- (void)connectAllAutoLoginServers
{
    for (id object in _arrayServers) {
        NSDebug(@"Inspecting Autologin for server: %@ (%@)", [object objectForKey:@"servername"], [object objectForKey:@"serviceName"]);
        BOOL theBool = [[object objectForKey:@"login"] boolValue];
        NSDebug(@"Autologin status: %@", [object objectForKey:@"login"]);
        if (theBool == YES) {
            
            NSString *serverVar;
            if ([[object objectForKey:@"servername"] hasPrefix:[@"smb://" lowercaseString]]) {
                serverVar = [[object objectForKey:@"servername"] substringFromIndex: 6];
            } else {
                serverVar = [object objectForKey:@"servername"];
            }
            
            if ([[object objectForKey:@"serviceName"] hasPrefix: serverVar]) {
                
                [self ConnectToServer:[object objectForKey:@"servername"] asUser:[object objectForKey:@"userID"] withPassword: RHKeychainGetGenericPassword(NULL, [object objectForKey:@"serviceName"])];
            }

        }
    }
}



@end
