//
//  ClickHandlerWindowController.m
//  SMB MenuExtra
//
//  Created by John Slaughter on 3/1/13.
//  Copyright (c) 2013 Engineering & Science Computing - University of Notre Dame.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "ClickHandlerWindowController.h"
#import "AppDelegate.h"
#import "SharedFunctions.h"
#import "AutoMountController.h"

@implementation ClickHandlerWindowController

@synthesize server = _server;
@synthesize user = _user;
@synthesize pass = _pass;
@synthesize statusOfWindow = _statusOfWindow;
@synthesize serverField = _serverField;
@synthesize userField = _userField;
@synthesize passField = _passField;
@synthesize cancelButton = _cancelButton;
@synthesize progressBar = _progressBar;
@synthesize protocolSelector = _protocolSelector;
@synthesize protocol = _protocol;


- (id) initWithServer:(NSString *)theServer
{
	self = [super init];
    if (self) {
        _server = theServer;
        
        if ( ! (self = [super initWithWindowNibName: @"ClickHandlerWindow"]) ) {
            return nil;
        }
    }
	return self;
}


- (void) awakeFromNib
{
    self.serverField.stringValue = self.server;
    NSProcessInfo *procOS = [NSProcessInfo processInfo];
	NSString *myOSVersion = [procOS operatingSystemVersionString];
    NSArray *myOSVersion_bits = [myOSVersion componentsSeparatedByString:@" "];
    if ([[myOSVersion_bits objectAtIndex:1] hasPrefix:@"10.9"]) {
        [protocolSelector setEnabled:NO];
    } else {
        [protocolSelector setEnabled:NO];
    }
    NSDebug(@"%@", [myOSVersion_bits objectAtIndex:1]);
    protocol = @"";
}


- (IBAction)closeWindow:(id)sender
{
    [self close];
}


- (IBAction)connectWindow:(id)sender
{
    [self.window makeFirstResponder: self.userField];

    [self.progressBar setUsesThreadedAnimation:YES];
    [self.progressBar startAnimation: self];
    
    NSDebug(@"Progress bar started.");
    
    AutoMountController *theController = [[AutoMountController alloc] init];
    
    NSDebug(@"AutoMountController created at: %@", [AutoMountController description]);

    [theController ConnectToServer: self.server asUser:[self.userField stringValue] withPassword:[self.passField stringValue]];
    
    NSDebug(@"Connecting to server: %@ as: %@", self.server, [self.userField stringValue]);
    
    [self.progressBar stopAnimation: self];
    [theController release];
    [self close];
}


- (IBAction)protocolChanged:(id)sender {
    
    switch ([[sender selectedCell] tag]) {
        case 1:
            NSDebug(@"SMB 1.x selected");
            protocol = @"cifs:";
            break;
        case 2:
            NSDebug(@"SMB 2.x selected");
            protocol = @"smb:";
            break;
        default:
            break;
    }
}


@end