//
//  ConnectWindowController.m
//  SMB MenuExtra
//
//  Created by John Slaughter on 3/1/13.
//  Copyright (c) 2013 Engineering & Science Computing - University of Notre Dame.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "ConnectWindowController.h"
#import "RHKeychain.h"
#import "AutoMountController.h"
#import "SharedFunctions.h"

@implementation ConnectWindowController

@synthesize defaults = _defaults;
@synthesize arrayServers = _arrayServers;
@synthesize tblServers = _tblServers;
@synthesize submitButton,
            cancelButton,
            userID,
            password,
            savePass,
            autoLogin,
            serverAddress,
            progressBar,
            protocolSelector,
            protocol;


#pragma mark - Standard Methods

- (id) init
{
	self = [super init];
    if (self) {
    
        // Init the User Defaults.
        _defaults = [NSUserDefaults standardUserDefaults];
            
        // Init the array of Servers.
        if ([self.defaults objectForKey:@"savedServers"] != nil) {
            _arrayServers  = [[NSMutableArray alloc] initWithArray:[_defaults arrayForKey:@"savedServers"]];
        } else {
            _arrayServers  = [[NSMutableArray alloc] init];
        }
        
        if ( ! (self = [super initWithWindowNibName: @"ConnectWindow"]) ) {
            return nil;
        }
    }
	return self;
}

- (void) awakeFromNib
{	
    // Make sure no tables rows are selected when window loads, and place ephesis on the server field.
    
    [self.tblServers deselectRow: [self.tblServers selectedRow]];
    [self.window makeFirstResponder: self.serverAddress];
    
    NSProcessInfo *procOS = [NSProcessInfo processInfo];
	NSString *myOSVersion = [procOS operatingSystemVersionString];
    NSArray *myOSVersion_bits = [myOSVersion componentsSeparatedByString:@" "];
    if ([[myOSVersion_bits objectAtIndex:1] hasPrefix:@"10.9"]) {
        
    } else {
        [protocolSelector setHidden:YES];
    }
    NSDebug(@"%@", [myOSVersion_bits objectAtIndex:1]);
    protocol = @"cifs:";
}


#pragma mark -  Methods


- (IBAction)closeTheWindow:(id)sender
{
	[self.defaults setObject:[self.serverAddress stringValue] forKey:@"serverAddress"];
	[self.defaults setObject:[self.userID stringValue] forKey:@"userID"];    
    if (self.arrayServers != nil) {
        [self.defaults setObject: self.arrayServers forKey:@"savedServers"];
	}
    [self.defaults synchronize];
    [self.tblServers deselectRow: [self.tblServers selectedRow]];
    [self.savePass setState: NSOffState];
    [self.password setStringValue:@""];
    [self.userID setStringValue:@""];
    [self.serverAddress setStringValue:@""];
    [self close];
}


#pragma mark - TableView Methods

- (IBAction)addServerToTable:(id)sender
{
    NSString *serviceName = @"";
    
    NSNumber *theState;
    if([self.savePass state] == NSOnState) {
        if ([[self.password stringValue] isNotEqualTo:@""]) {
            
            int RandomIndex = arc4random() % 10000000;
            serviceName = [NSString stringWithFormat:@"%@-%@-%d", removeSMBPrefix([self.serverAddress stringValue]), [self.userID stringValue], RandomIndex];
            
            if (!RHKeychainDoesGenericEntryExist(NULL, serviceName)) {
                RHKeychainAddGenericEntry(NULL, serviceName);
                RHKeychainSetGenericUsername(NULL, serviceName, [self.userID stringValue]);
                RHKeychainSetGenericPassword(NULL, serviceName, [self.password stringValue]);
                RHKeychainSetGenericComment(NULL, serviceName, [self.serverAddress stringValue]);
            }
            
            theState = [NSNumber numberWithBool:YES];
        } else {
            NSAlert * alert = [NSAlert alertWithMessageText:@"Invalid Password" defaultButton:@"Try Again" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Did you forget to enter a password? You asked me to save it, but it appears to be blank."];
            [alert runModal];
            return;
        }
    } else {
        theState = [NSNumber numberWithBool:NO];
    }
    
    NSNumber *theLogin = [NSNumber numberWithBool:NO];

    
    NSMutableDictionary *dict =[[[NSMutableDictionary alloc] initWithObjectsAndKeys:[self.serverAddress stringValue], @"servername", [self.userID stringValue], @"userID", theState, @"savePass", theLogin, @"login", serviceName, @"serviceName", nil] autorelease];
    
    NSDebug(@"%@", [dict description]);
    
	[self.arrayServers addObject:dict];
	[self.tblServers reloadData];
}

- (IBAction)removeServerFromTable:(id)sender
{
	if ([self.tblServers selectedRow] < 0 ) {
		NSBeep();
	}
    if ([self.tblServers selectedRow] < 0 || [self.tblServers selectedRow] >= [self.arrayServers count]) {
        return;
    }
    
    if ([[self.arrayServers objectAtIndex:[self.tblServers selectedRow]] objectForKey:@"serviceName"]) {

        NSString *serviceName = [[self.arrayServers objectAtIndex:[self.tblServers selectedRow]] objectForKey:@"serviceName"];
        
        if (RHKeychainDoesGenericEntryExist(NULL, serviceName)) {
            RHKeychainRemoveGenericEntry(NULL, serviceName);
            if (RHKeychainDoesGenericEntryExist(NULL, serviceName)) {
                NSDebug(@"Keychain Password removal failed.");
            } else {
                NSDebug(@"Keychain Password removal succeeded.");
            }
        }
    }
    [self.arrayServers removeObjectAtIndex:[self.tblServers selectedRow]];
    [self.tblServers reloadData];
}


#pragma mark - Connection Methods

- (IBAction)ConnectToShare:(id)sender
{
    [self.progressBar setUsesThreadedAnimation:YES];
    [self.progressBar startAnimation: self];
    
    int RandomIndex = arc4random() % 100000;
    NSString * tempPath = [NSTemporaryDirectory() stringByAppendingPathComponent: [NSString stringWithFormat:@"Share_%d", RandomIndex]];
    NSDebug(@"tempPath: %@", tempPath);
    
    NSTask *makeTmp=[[[NSTask alloc] init] autorelease];
    [makeTmp setLaunchPath: @"/bin/mkdir"];
    [makeTmp setArguments:[NSArray arrayWithObjects:tempPath, nil]];
    [makeTmp launch];
    
    NSDebug(@"tempPath created: %@", tempPath);
    NSDebug(@"Connecting to SMB using protocol: '%@'", protocol);
    
    NSTask *mountFS=[[NSTask alloc] init];
    [mountFS setLaunchPath: @"/sbin/mount"];
    [mountFS setArguments:[NSArray arrayWithObjects: @"-v", @"-t", @"smbfs", [NSString stringWithFormat:@"%@//%@:%@@%@", protocol, sanitizeString([self.userID stringValue]), sanitizeString([self.password stringValue]), sanitizeString(removeSMBPrefix([self.serverAddress stringValue]))], tempPath, nil]];
    
    NSPipe *pipeEthernet = [NSPipe pipe];
    [mountFS setStandardOutput: pipeEthernet];
    [mountFS setStandardError: pipeEthernet];
    [mountFS launch];
    [mountFS waitUntilExit];
    
    NSData *outputData = [[pipeEthernet fileHandleForReading] readDataToEndOfFile];
    NSString *outputString = [[NSString alloc] initWithData:outputData encoding:NSUTF8StringEncoding];
    NSDebug(@"NSTask Output: %@", outputString);
    
    
    [self.defaults setObject:[self.serverAddress stringValue] forKey:@"serverAddress"];
    [self.defaults setObject:[self.userID stringValue] forKey:@"userID"];
    if (self.arrayServers != nil) {
        [self.defaults setObject: self.arrayServers forKey:@"savedServers"];
    }
    [self.defaults synchronize];
    [self.password setStringValue:@""];
    [self.progressBar stopAnimation: self];
    
    if ([outputString rangeOfString:@"Operation timed out"].location != NSNotFound) {
        NSDebug(@"Connect Result: Operation timed out");
        NSAlert * alert = [NSAlert alertWithMessageText:@"Server Timeout" defaultButton:@"Cancel" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Could not contact server.  Please verify your internet connection and try again."];
        [alert runModal];
    } else if ([outputString rangeOfString:@"No route to host"].location != NSNotFound) {
        NSDebug(@"Connect Result: No route to host");
        NSAlert * alert = [NSAlert alertWithMessageText:@"Connection Failed" defaultButton:@"Cancel" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Could not contact server.  Please verify your internet connection and try again."];
        [alert runModal];
    } else if ([outputString rangeOfString:@"No such file or directory"].location != NSNotFound) {
        NSDebug(@"Connect Result: No such file or directory");
        NSAlert * alert = [NSAlert alertWithMessageText:@"Server Path Error" defaultButton:@"Cancel" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Server returned 'No such file or directory.'  Please verify the server path and try again."];
        [alert runModal];
    } else if ([outputString rangeOfString:@"Authentication error"].location != NSNotFound) {
        NSDebug(@"Connect Result: Authentication error");
        NSAlert * alert = [NSAlert alertWithMessageText:@"Authentication Error" defaultButton:@"Cancel" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Server returned: 'Authentication error.'  Please verify your username or password and try again."];
        [alert runModal];
    } else {
        [self close];
        [[NSWorkspace sharedWorkspace] openFile:tempPath ];
    }
    
    [outputString release];
    [mountFS release];
}


- (IBAction)mountAllSavedVolumes:(id)sender
{
    // Currently unused in this interface.  The window just got too cluttered.
    
    [self.progressBar setUsesThreadedAnimation:YES];
    [self.progressBar startAnimation: self];
    
    AutoMountController *theAutoMount = [[AutoMountController alloc] init];
    
    [theAutoMount connectAllSavedServers];
    
    [theAutoMount release];
    
    [self.progressBar stopAnimation: self];
    [self close];
}


- (IBAction)mountAllCheckedVolumes:(id)sender
{
    // Currently unused in this interface.  The window just got too cluttered.
    
    [self.progressBar setUsesThreadedAnimation:YES];
    [self.progressBar startAnimation: self];
    
    AutoMountController *theAutoMount = [[AutoMountController alloc] init];
    
    [theAutoMount connectAllAutoLoginServers];
    
    [theAutoMount release];
    
    [self.progressBar stopAnimation: self];
    [self close];
}


#pragma mark - TableView Methods

- (NSInteger)numberOfRowsInTableView:(NSTableView *)tv
{
	return (NSInteger)[self.arrayServers count];
}


- (id)tableView:(NSTableView *)tv objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{   
    if ([[tableColumn identifier] isEqualToString:@"userID"]) {
        if ([[self.arrayServers objectAtIndex:row] objectForKey:@"userID"]) {
            return [[self.arrayServers objectAtIndex:row] objectForKey:@"userID"];
        } else {
            return @"";
        }
    } else if ([[tableColumn identifier] isEqualToString:@"savePass"]) {
        if ([[self.arrayServers objectAtIndex:row] objectForKey:@"savePass"]) {
            BOOL theBool = [[[self.arrayServers objectAtIndex:row] objectForKey:@"savePass"] boolValue];
            if (theBool) {
                //return @"Y";
                return([NSImage imageNamed:@"lock.png"]);
            } else {
                //return @"";
                return([NSImage imageNamed:@"fail.png"]);
            }
        } else {
            //return @"";
            return([NSImage imageNamed:@"fail.png"]);

        }
    } else if ([[tableColumn identifier] isEqualToString:@"login"]) {
        if ([[self.arrayServers objectAtIndex:row] objectForKey:@"login"]) {
            BOOL theBool = [[[self.arrayServers objectAtIndex:row] objectForKey:@"login"] boolValue];
            return [NSNumber numberWithInteger:(theBool ? NSOnState : NSOffState)];
        } else {
            //return @"";
            return [NSNumber numberWithInteger:NSOffState];
        }
    } else {
        return [[self.arrayServers objectAtIndex:row] objectForKey:[tableColumn identifier]];
    }
}

#pragma mark - NSTableViewDelegate

- (void)tableViewSelectionDidChange:(NSNotification *)notification
{
	NSInteger row = [self.tblServers selectedRow];
	if (row == -1) {
		return;
    }
    
    [self.serverAddress setStringValue: [[self.arrayServers objectAtIndex:row] objectForKey:@"servername"]];
    
    if ([[self.arrayServers objectAtIndex:row] objectForKey:@"userID"]) {
        [self.userID setStringValue: [[self.arrayServers objectAtIndex:row] objectForKey:@"userID"]];
    } else {
        [self.userID setStringValue: @""];
    }
    
    if ([[self.arrayServers objectAtIndex:row] objectForKey:@"savePass"]) {
        int theValue = [[[self.arrayServers objectAtIndex:row] objectForKey:@"savePass"] intValue];
        if (theValue == 1) {
            [self.savePass setState: NSOnState];
            
            if ([[self.arrayServers objectAtIndex:row] objectForKey:@"serviceName"]) {
                NSString *serviceName = [[self.arrayServers objectAtIndex:row] objectForKey:@"serviceName"];
                
                NSString *serverVar;
                if ([[[self.arrayServers objectAtIndex:row] objectForKey:@"servername"] hasPrefix:[@"smb://" lowercaseString]]) {
                    serverVar = [[[self.arrayServers objectAtIndex:row] objectForKey:@"servername"] substringFromIndex: 6];
                } else {
                    serverVar = [[self.arrayServers objectAtIndex:row] objectForKey:@"servername"];
                }
                
                if ([serviceName hasPrefix: serverVar]) {
                    if (RHKeychainDoesGenericEntryExist(NULL, serviceName)) {
                        NSString *tmpPass = RHKeychainGetGenericPassword(NULL, serviceName);
                        if (tmpPass != nil) {
                            [self.password setStringValue: RHKeychainGetGenericPassword(NULL, serviceName)];
                        } else {
                            [self.password setStringValue:@""];
                        }
                    }
            }
            } else {
                [self.savePass setState: NSOffState];
                [self.password setStringValue:@""];
            }
        } else {
            [self.savePass setState: NSOffState];
            [self.password setStringValue:@""];
        }
    } else {
        [self.savePass setState: NSOffState];
        [self.password setStringValue:@""];
    }
    
}


- (void)tableView:(NSTableView *)tableView setObjectValue:(id)value forTableColumn:(NSTableColumn *)column row:(NSInteger)row {
  //  [[myRows objectAtIndex:row] setBooleanAttribute:[value booleanValue]];
    
    NSString *theServer     = [[self.arrayServers objectAtIndex:row] objectForKey:@"servername"];
    NSString *theuserID      = [[self.arrayServers objectAtIndex:row] objectForKey:@"userID"];
    NSNumber *theSavePass   = [[self.arrayServers objectAtIndex:row] objectForKey:@"savePass"];
	NSNumber *theLogin      = [[self.arrayServers objectAtIndex:row] objectForKey:@"login"];
    NSNumber *theService    = [[self.arrayServers objectAtIndex:row] objectForKey:@"serviceName"];

    
    if ([[column identifier] isEqualToString:@"userID"]) {
        // theuserID = value;
    } else if ([[column identifier] isEqualToString:@"login"]) {
        int theValue = [[[self.arrayServers objectAtIndex:row] objectForKey:@"savePass"] intValue];
        if (theValue == 1) {
            theLogin = [NSNumber numberWithBool:[value boolValue]];
        } else {
            theLogin = [NSNumber numberWithBool:NO];
        }
    }
    
    NSMutableDictionary *dict =[[[NSMutableDictionary alloc] initWithObjectsAndKeys:theServer, @"servername", theuserID, @"userID", theSavePass, @"savePass", theLogin, @"login", theService, @"serviceName", nil] autorelease];
    
    NSDebug(@"%@", [dict description]);
    
	[self.arrayServers replaceObjectAtIndex:row withObject: dict];
    [self.defaults setObject: self.arrayServers forKey:@"savedServers"];
    [self.defaults synchronize];
}

- (IBAction)protocolChanged:(id)sender {
    
    switch ([[sender selectedCell] tag]) {
        case 1:
            NSDebug(@"SMB 1.x selected");
            protocol = @"cifs:";
            break;
        case 2:
            NSDebug(@"SMB 2.x selected");
            protocol = @"smb:";
            break;
        default:
            break;
    }
}



@end