//
//  ShowServersOnDesktop.m
//  SMB MenuExtra
//
//  Created by John Slaughter on 3/17/13.
//  Copyright (c) 2013 Engineering & Science Computing - University of Notre Dame.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//
#import "ShowServersOnDesktop.h"

@implementation ShowServersOnDesktop

- (void) setShowServersOnDesktop: (BOOL) enabled
{
    // defaults write com.apple.finder ShowMountedServersOnDesktop

    NSDebug(@"setShowServersOnDesktop: '%d'", enabled);
    
    NSString *stringBool = @"false";
    if (enabled == 0) {
        stringBool = @"false";
    } else if (enabled == 1) {
        stringBool = @"true";
    }
    
    NSTask *writeServerSetting=[[NSTask alloc] init];
    [writeServerSetting setLaunchPath: @"/usr/bin/defaults"];
    [writeServerSetting setArguments:[NSArray arrayWithObjects:@"write", @"com.apple.finder", @"ShowMountedServersOnDesktop", @"-boolean", stringBool, nil]];
    
    NSPipe *pipeSetting = [NSPipe pipe];
    [writeServerSetting setStandardOutput: pipeSetting];
    [writeServerSetting setStandardError: pipeSetting];
    [writeServerSetting launch];
    
    [writeServerSetting waitUntilExit];
    
    NSData *outputData = [[pipeSetting fileHandleForReading] readDataToEndOfFile];
    NSString *outputString = [[NSString alloc] initWithData:outputData encoding:NSUTF8StringEncoding];
    NSDebug(@"NSTask Output: %@", outputString);
    
    [outputString release];
    [writeServerSetting release];

}


- (BOOL) showServersOnDesktop
{
    return [self readServerOnDesktopSetting];
}


- (BOOL)readServerOnDesktopSetting
{
    // defaults read com.apple.finder ShowMountedServersOnDesktop
    
    BOOL theBool = FALSE;
    
    NSTask *readServerSetting=[[NSTask alloc] init];
    [readServerSetting setLaunchPath: @"/usr/bin/defaults"];
    [readServerSetting setArguments:[NSArray arrayWithObjects:@"read", @"com.apple.finder", @"ShowMountedServersOnDesktop", nil]];
    
    NSPipe *pipeSetting = [NSPipe pipe];
    [readServerSetting setStandardOutput: pipeSetting];
    [readServerSetting setStandardError: pipeSetting];
    [readServerSetting launch];
    
    [readServerSetting waitUntilExit];
    
    NSData *outputData = [[pipeSetting fileHandleForReading] readDataToEndOfFile];
    NSString *outputString = [[NSString alloc] initWithData:outputData encoding:NSUTF8StringEncoding];
    NSDebug(@"NSTask Output: %@", outputString);
    
    
    if ([outputString isEqualToString:@"0\n"]) {
        NSDebug(@"Server on Desktop is Bool FALSE");
        theBool = FALSE;
    } else if ([outputString isEqualToString:@"1\n"]) {
        NSDebug(@"Server on Desktop is Bool TRUE");
        theBool = TRUE;
    }
    
    [outputString release];
    [readServerSetting release];
    return theBool;
}


@end